#include "cuda_fx.h"

/*
 * The main function that runs the CCI tests using both the simple and the optimized implementations and outputs comparison information
 */
void runCci(float* highValues, float* lowValues, float* closeValues, float* spreadValues, float* d_highValues, float* d_lowValues, float* d_closeValues, float* d_spreadValues, int nRecords, int takePositionAfterNRecords, int minN, int maxN, int minUpperBreakout, int maxUpperBreakout, int minLowerBreakout, int maxLowerBreakout) {
	// Timer variables
	float tmrCPU1 = 0.0f, tmrCPU2 = 0.0f;

	// Output variables
	int chosenNCpu2 = 0, chosenNCpu1 = 0, chosenUpperBreakoutCpu2 = 0, chosenUpperBreakoutCpu1 = 0, chosenLowerBreakoutCpu2 = 0, chosenLowerBreakoutCpu1 = 0;
	float profitCpu2 = 0.0f, profitCpu1 = 0.0f;
	
	// *********** CCI *************
	// *** CPU2 Algorithm ***
	M_timedCpuCall(tmrCPU2, cpuCciMain2(highValues, lowValues, closeValues, spreadValues, nRecords, takePositionAfterNRecords, minN, maxN, minUpperBreakout, maxUpperBreakout, minLowerBreakout, maxLowerBreakout, chosenNCpu2, chosenUpperBreakoutCpu2, chosenLowerBreakoutCpu2, profitCpu2));

	// *** CPU1 algorithm ***
	M_timedCpuCall(tmrCPU1, cpuCciMain(highValues, lowValues, closeValues, spreadValues, nRecords, takePositionAfterNRecords, minN, maxN, minUpperBreakout, maxUpperBreakout, minLowerBreakout, maxLowerBreakout, chosenNCpu1, chosenUpperBreakoutCpu1, chosenLowerBreakoutCpu1, profitCpu1));
	
	// Compare CPU2/CPU1 output
	printf("\n***CCI Results:***\n");
	printf("CPU2 Max MA: %d\tCPU1 Max MA: %d\n", chosenNCpu2, chosenNCpu1);
	printf("CPU2 Max Upper: %d\tCPU1 Max Upper: %d\n", chosenUpperBreakoutCpu2, chosenUpperBreakoutCpu1);
	printf("CPU2 Max Lower: %d\tCPU1 Max Lower: %d\n", chosenLowerBreakoutCpu2, chosenLowerBreakoutCpu1);
	printf("CPU2 Profit: %f\tCPU1 Profit: %f\n", profitCpu2, profitCpu1);
	printf("CPU2 Time: %f\tCPU1 Time: %f\tCPU2 Speedup: %fx\n", tmrCPU2, tmrCPU1, (tmrCPU1 / tmrCPU2));

	//if (chosenNCpu2 != chosenNCpu1 || abs(profitCpu2 - profitCpu1)>.000001)
	if (abs(profitCpu2 - profitCpu1)>.000001)
		printf("-- Result mismatch detected. --\n\n");
}
