#include "cuda_fx.h"

/*
 * The main function that runs the EMA tests on CPU and GPU and outputs comparison information
 */
void runEma (float* openValues, float *closeValues, float *spreadValues, float* d_openValues, float* d_closeValues, float* d_spreadValues, int nRecords, int takePositionAfterNRecords, int minN, int maxN) {  
	// Timer variables
	float tmrCPU = 0.0f, tmrGPU = 0.0f;

	// Output variables
	int chosenNGpu = 0, chosenNCpu = 0, chosenLongNGpu = 0, chosenLongNCpu = 0, chosenShortNGpu = 0, chosenShortNCpu = 0;
	float profitGpu = 0.0f, profitCpu = 0.0f;
	
	// *********** EMA-S *************
	// *** GPU Algorithm ***
	M_timedCpuCall(tmrGPU, gpuEmaMain(false, d_openValues, d_closeValues, d_spreadValues, nRecords, takePositionAfterNRecords, minN, maxN, chosenNGpu, profitGpu));

	// *** CPU algorithm ***
	M_timedCpuCall(tmrCPU, cpuEmaMain(false, openValues, closeValues, spreadValues, nRecords, takePositionAfterNRecords, minN, maxN, chosenNCpu, profitCpu));

	// Compare GPU/CPU output
	printf("\n***EMA-S Results:***\n");
	printf("GPU Max: %d\t\tCPU Max: %d\n", chosenNGpu, chosenNCpu);
	printf("GPU Profit: %f\tCPU Profit: %f\n", profitGpu, profitCpu);
	printf("GPU Time: %f\tCPU Time: %f\tGPU Speedup: %fx\n", tmrGPU, tmrCPU, (tmrCPU / tmrGPU));

	//if (chosenNGpu != chosenNCpu || abs(profitGpu - profitCpu)>.000001)
	if (abs(profitGpu - profitCpu)>.000001)
		printf("-- Result mismatch detected. --\n\n");

	// *********** EMA-OC *************
	chosenNGpu = 0;
	chosenNCpu = 0;
	profitCpu = 0;
	profitGpu = 0;

	// *** GPU Algorithm ***
	M_timedCpuCall(tmrGPU, gpuEmaMain(true, d_openValues, d_closeValues, d_spreadValues, nRecords, takePositionAfterNRecords, minN, maxN, chosenNGpu, profitGpu));

	// *** CPU algorithm ***
	M_timedCpuCall(tmrCPU, cpuEmaMain(true, openValues, closeValues, spreadValues, nRecords, takePositionAfterNRecords, minN, maxN, chosenNCpu, profitCpu));

	// Compare GPU/CPU output
	printf("\n***EMA-OC Results:***\n");
	printf("GPU Max: %d\t\tCPU Max: %d\n", chosenNGpu, chosenNCpu);
	printf("GPU Profit: %f\tCPU Profit: %f\n", profitGpu, profitCpu);
	printf("GPU Time: %f\tCPU Time: %f\tGPU Speedup: %fx\n", tmrGPU, tmrCPU, (tmrCPU / tmrGPU));

	//if (chosenNGpu != chosenNCpu || abs(profitGpu - profitCpu)>.000001)
	if (abs(profitGpu - profitCpu)>.000001)
		printf("-- Result mismatch detected. --\n\n");


	// *********** EMA-D *************
	// *** GPU Algorithm ***
	M_timedCpuCall(tmrGPU, gpuDoubleEmaMain(d_closeValues, d_spreadValues, nRecords, takePositionAfterNRecords, minN, maxN, chosenLongNGpu, chosenShortNGpu, profitGpu));

	// *** CPU algorithm ***
	M_timedCpuCall(tmrCPU, cpuDoubleEmaMain(closeValues, spreadValues, nRecords, takePositionAfterNRecords, minN, maxN, chosenLongNCpu, chosenShortNCpu, profitCpu));

	// Compare GPU/CPU output
	printf("\n***EMA-D Results:***\n");
	printf("GPU Max Long: %d\tCPU Max Long: %d\n", chosenLongNGpu, chosenLongNCpu);
	printf("GPU Max Short: %d\tCPU Max Short: %d\n", chosenShortNGpu, chosenShortNCpu);
	printf("GPU Profit: %f\tCPU Profit: %f\n", profitGpu, profitCpu);
	printf("GPU Time: %f\tCPU Time: %f\tGPU Speedup: %fx\n", tmrGPU, tmrCPU, (tmrCPU / tmrGPU));

	//if (chosenLongNGpu != chosenLongNCpu || chosenShortNGpu != chosenShortNCpu || abs(profitGpu - profitCpu)>.000001)
	if (abs(profitGpu - profitCpu)>.000001)
		printf("-- Result mismatch detected. --\n\n");
}
