#include "cuda_fx.h"

/*
 * The main function that runs the MACD tests on CPU and GPU and outputs comparison information
 */
void runMacd (float *closeValues, float *spreadValues, float* d_closeValues, float* d_spreadValues, int nRecords, int takePositionAfterNRecords, int minN, int maxN) {  
	// Timer variables
	float tmrCPU = 0.0f, tmrGPU = 0.0f;

	// Output variables
	int chosenLongNGpu = 0, chosenLongNCpu = 0, chosenShortNGpu = 0, chosenShortNCpu = 0, chosenSignalNCpu = 0, chosenSignalNGpu = 0;
	float profitGpu = 0.0f, profitCpu = 0.0f;
	
	// *********** MACD *************
	// *** GPU Algorithm ***
	M_timedCpuCall(tmrGPU, gpuMacdMain(d_closeValues, d_spreadValues, nRecords, takePositionAfterNRecords, minN, maxN, chosenLongNGpu, chosenShortNGpu, chosenSignalNGpu, profitGpu));

	// *** CPU algorithm ***
	M_timedCpuCall(tmrCPU, cpuMacdMain(closeValues, spreadValues, nRecords, takePositionAfterNRecords, minN, maxN, chosenLongNCpu, chosenShortNCpu, chosenSignalNCpu, profitCpu));

	// Compare GPU/CPU output
	printf("\n***MACD Results:***\n");
	printf("GPU Max Long: %d\tCPU Max Long: %d\n", chosenLongNGpu, chosenLongNCpu);
	printf("GPU Max Short: %d\tCPU Max Short: %d\n", chosenShortNGpu, chosenShortNCpu);
	printf("GPU Max Signal: %d\tCPU Max Signal: %d\n", chosenSignalNGpu, chosenSignalNCpu);
	printf("GPU Profit: %f\tCPU Profit: %f\n", profitGpu, profitCpu);
	printf("GPU Time: %f\tCPU Time: %f\tGPU Speedup: %fx\n", tmrGPU, tmrCPU, (tmrCPU / tmrGPU));

	if (abs(profitGpu - profitCpu)>.000001)
		printf("-- Result mismatch detected. --\n\n");
}
