#include "cuda_fx.h"

#include "EmaMain.cu"
#include "cpuEma.cu"
#include "gpuEma.cu"
#include "MacdMain.cu"
#include "cpuMacd.cu"
#include "gpuMacd.cu"
#include "CciMain.cu"
#include "cpuCci.cu"
#include "cpuCci2.cu"
#include "ArgMax.cu"

// Include samples (data points) for analysis
//#include "samples/AUD_JPY_5min.c"
//#include "samples/AUD_USD_5min.c"
#include "samples/EUR_USD_5min.c"
//#include "samples/USD_CAD_5min.c"

//#include "samples/AUD_JPY_daily.c"
//#include "samples/AUD_USD_daily.c"
//#include "samples/EUR_USD_daily.c"
//#include "samples/USD_CAD_daily.c"

////////////////////////////////////////////////////////////////////////////////
// Program main
////////////////////////////////////////////////////////////////////////////////
int main (int argc, char ** argv) {
	runAnalysis(NULL, NULL, NULL, NULL, NULL, 0, 0);
}


//void runAnalysis (float* h_arrOpen, float* h_arrHigh, float* h_arrLow, float* h_arrClose, float* h_arrSpread, __time64_t* h_arrDate, int intTotalSampleCount, int intSamplesToAnalyze) {
void runAnalysis (float* h_arrOpen, float* h_arrHigh, float* h_arrLow, float* h_arrClose, float* h_arrSpread, int intTotalSampleCount, int intSamplesToAnalyze) {
	// Initialize data points array if this function was not called directly via DLL call
	if (h_arrOpen == NULL) {
		// The sample count is H + N (includes historic data)
		intTotalSampleCount = getTotalSampleCount();
		// The number of samples to analyze is N
		intSamplesToAnalyze = getAnalysisSampleCount();
		h_arrOpen = (float *) malloc (sizeof(float) * intTotalSampleCount);
		h_arrHigh = (float *) malloc (sizeof(float) * intTotalSampleCount);
		h_arrLow = (float *) malloc (sizeof(float) * intTotalSampleCount);
		h_arrClose = (float *) malloc (sizeof(float) * intTotalSampleCount);
		h_arrSpread = (float *) malloc (sizeof(float) * intTotalSampleCount);
		//h_arrDate = (__time64_t *) malloc (sizeof(__time64_t) * intTotalSampleCount);
		initSamples(h_arrOpen, h_arrHigh, h_arrLow, h_arrClose, h_arrSpread);
	}

	int intFloatArraySize = sizeof(float) * intTotalSampleCount;
	//int intDateArraySize = sizeof(__time64_t) * intTotalSampleCount;


	// Copy datapoints to device
    float *d_arrOpen, *d_arrHigh, *d_arrLow, *d_arrClose, *d_arrSpread;
	//__time64_t *d_arrDate;
    CUDA_SAFE_CALL( cudaMalloc( (void**) &d_arrOpen, intFloatArraySize));
	CUDA_SAFE_CALL( cudaMalloc( (void**) &d_arrHigh, intFloatArraySize));
	CUDA_SAFE_CALL( cudaMalloc( (void**) &d_arrLow, intFloatArraySize));
	CUDA_SAFE_CALL( cudaMalloc( (void**) &d_arrClose, intFloatArraySize));
	CUDA_SAFE_CALL( cudaMalloc( (void**) &d_arrSpread, intFloatArraySize));
	//CUDA_SAFE_CALL( cudaMalloc( (void**) &d_arrDate, intDateArraySize));

    CUDA_SAFE_CALL( cudaMemcpy( d_arrOpen, h_arrOpen, intFloatArraySize, cudaMemcpyHostToDevice) );
	CUDA_SAFE_CALL( cudaMemcpy( d_arrHigh, h_arrHigh, intFloatArraySize, cudaMemcpyHostToDevice) );
	CUDA_SAFE_CALL( cudaMemcpy( d_arrLow, h_arrLow, intFloatArraySize, cudaMemcpyHostToDevice) );
	CUDA_SAFE_CALL( cudaMemcpy( d_arrClose, h_arrClose, intFloatArraySize, cudaMemcpyHostToDevice) );
	CUDA_SAFE_CALL( cudaMemcpy( d_arrSpread, h_arrSpread, intFloatArraySize, cudaMemcpyHostToDevice) );
	//CUDA_SAFE_CALL( cudaMemcpy( d_arrDate, h_arrDate, intDateArraySize, cudaMemcpyHostToDevice) );

	// Ensure copy completed
	cudaThreadSynchronize ();

	/************ Exponential Moving Averages ****************/
	// Call EMA main function
	runEma (h_arrOpen, h_arrClose, h_arrSpread, d_arrOpen, d_arrClose, d_arrSpread, intTotalSampleCount, intTotalSampleCount - intSamplesToAnalyze, 2, intSamplesToAnalyze - 1);

	/************ Moving Average Convergence Divergence ****************/
	// Call MACD main function
	runMacd (h_arrClose, h_arrSpread, d_arrClose, d_arrSpread, intTotalSampleCount, intTotalSampleCount - intSamplesToAnalyze, 2, intSamplesToAnalyze - 1);

	/************ Commodity Channel Index ****************/
	// Call CCI main function
	int minUpperBreakout = 30, maxUpperBreakout = 300;
	int minLowerBreakout = -minUpperBreakout, maxLowerBreakout = -maxUpperBreakout;
	runCci (h_arrHigh, h_arrLow, h_arrClose, h_arrSpread, d_arrHigh, d_arrLow, d_arrClose, d_arrSpread, intTotalSampleCount, intTotalSampleCount - intSamplesToAnalyze, 2, intSamplesToAnalyze - 1, minUpperBreakout, maxUpperBreakout, minLowerBreakout, maxLowerBreakout);
}


#if !defined (_WIN32)
// Returns the time interval in milliseconds between the two times specified
float getTimeInterval (struct timeval tmrEnd, struct timeval tmrStart) {
  return (tmrEnd.tv_sec - tmrStart.tv_sec + tmrEnd.tv_usec/1e6 - tmrStart.tv_usec/1e6) * 1e3;
}
#endif

