#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <sys/types.h>


#if defined (_WIN32)
#include "windows.h"
#else
#include <sys/time.h>
#endif

#include <cutil.h>



#define M_repI(nTimes) for (int i=0; i<nTimes; i++) 

#define M_repI2(from, upTo) for (int i=from; i<upTo; i++) 

#define M_cudaCall(command) do {                            \
  command;                                                  \
  cudaThreadSynchronize();                                  \
  CUT_CHECK_ERROR ("kernel failed");                        \
  } while (0);

#define M_timedCudaCall(timeVariable, command) do {	\
  unsigned int timerVariable;                                                            \
  cutCreateTimer(&timerVariable);                                                        \
  cudaThreadSynchronize();                                                               \
  cutStartTimer(timerVariable);                                                          \
  M_cudaCall(command);							                 \
  cutStopTimer(timerVariable);                                                           \
  timeVariable = cutGetTimerValue(timerVariable);                                        \
  cutDeleteTimer(timerVariable);                                                         \
  } while (0);

#if defined (_WIN32)
#define M_timedCpuCall(timeVariable, command) do { \
	LARGE_INTEGER ticksPerSecond;																			\
	LARGE_INTEGER starttime;   																				\
	LARGE_INTEGER endtime;   																				\
	QueryPerformanceFrequency(&ticksPerSecond);																\
	QueryPerformanceCounter(&starttime);																	\
	command;																								\
	QueryPerformanceCounter(&endtime);																		\
	timeVariable = (float) ((endtime.QuadPart - starttime.QuadPart) * 1000 / (double) ticksPerSecond.QuadPart);	\
	} while (0);
#else
#define M_timedCpuCall(timeVariable, command) do { \
	struct timeval starttime;															\
	struct timeval endtime;															\
	gettimeofday(&starttime, NULL);																\
	command;																			\
	gettimeofday (&endtime, NULL);																\
	timeVariable = getTimeInterval(endtime, starttime);									\
	} while (0);
#endif

#define M_cudaMalloc2(pointerVariable, type, nElements) 	                   \
  type* pointerVariable;                                                           \
  M_cudaCall(cudaMalloc((void**)&pointerVariable, nElements*sizeof(type)));




// Analysis function prototypes
//void runAnalysis(float*, float* h_arrHigh, float* h_arrLow, float* h_arrClose, float* h_arrSpread, __time64_t* h_arrDate, int, int);
void runAnalysis(float*, float* h_arrHigh, float* h_arrLow, float* h_arrClose, float* h_arrSpread, int, int);


// Sample function prototypes
extern int getTotalSampleCount();
extern int getAnalysisSampleCount();
//extern void initSamples(float* h_arrOpen, float* h_arrHigh, float* h_arrLow, float* h_arrClose, float* h_arrSpread, __time64_t* h_arrDate);
extern void initSamples(float* h_arrOpen, float* h_arrHigh, float* h_arrLow, float* h_arrClose, float* h_arrSpread);

// EMA prototypes
void runEma (float* openValues, float *closeValues, float *spreadValues, float* d_openValues, float* d_closeValues, float* d_spreadValues, int nRecords, int takePositionAfterNRecords, int minN, int maxN);
void gpuEmaMain(bool checkOC, float* d_openValues, float* d_closeValues, float* d_spreadValues, int nRecords, int takePositionAfterNRecords, int minN, int maxN, int& chosenN, float& profit);
void gpuDoubleEmaMain(float* d_closeValues, float* d_spreadValues, int nRecords, int takePositionAfterNRecords, int minN, int maxN, int& chosenLongN, int& chosenShortN, float& profit);
void cpuEmaMain(bool checkOC, float* openValues, float* d_closeValues, float* d_spreadValues, int nRecords, int takePositionAfterNRecords, int minN, int maxN, int& chosenN, float& profit);
void cpuDoubleEmaMain(float* closeValues, float* spreadValues, int nRecords, int takePositionAfterNRecords, int minN, int maxN, int& chosenLongN, int& chosenShortN, float& profit);

// MACD prototypes
void runMacd(float* closeValues, float* spreadValues, float* d_closeValues, float* d_spreadValues, int nRecords, int takePositionAfterNRecords, int minN, int maxN);
void gpuMacdMain(float* d_closeValues, float* d_spreadValues, int nRecords, int takePositionAfterNRecords, int minN, int maxN, int& chosenLongN, int& chosenShortN, int& chosenSignalN, float& profit);
void cpuMacdMain(float* closeValues, float* spreadValues, int nRecords, int takePositionAfterNRecords, int minN, int maxN, int& chosenLongN, int& chosenShortN, int& chosenSignalN, float& profit);

// CCI prototypes
void runCci(float* highValues, float* lowValues, float* closeValues, float* spreadValues, float* d_highValues, float* d_lowValues, float* d_closeValues, float* d_spreadValues, int nRecords, int takePositionAfterNRecords, int minN, int maxN, int minUpperBreakout, int maxUpperBreakout, int minLowerBreakout, int maxLowerBreakout);
void cpuCciMain(float* highValues, float* lowValues, float* closeValues, float* spreadValues, int nRecords, int takePositionAfterNRecords, int minN, int maxN, int minUpperBreakout, int maxUpperBreakout, int minLowerBreakout, int maxLowerBreakout, int& chosenN, int& chosenUpperBreakout, int& chosenLowerBreakout, float& profit);
void cpuCciMain2(float* highValues, float* lowValues, float* closeValues, float* spreadValues, int nRecords, int takePositionAfterNRecords, int minN, int maxN, int minUpperBreakout, int maxUpperBreakout, int minLowerBreakout, int maxLowerBreakout, int& chosenN, int& chosenUpperBreakout, int& chosenLowerBreakout, float& profit);

// Arg max prototypes
int cpuArgmax(float* values, int nValues);
int gpuArgmax(float* d_values, int nValues);

// Time interval prototypes
#if !defined (_WIN32)
float getTimeInterval (struct timeval tmrEnd, struct timeval tmrStart);
#endif